@@ -1,19 +1,12 @@
-/*  Copyright (c) 2010 Robin Sonefors
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
- *
- *  This program is distributed in the hope that it will be useful, but
- *  WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- *  General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- *  02111-1307, USA.
+/*
+ * Copyright (C) 2010 Robin Sonefors
+ * Copyright (C) 2008-2011 Robert Ancell.
+ * 
+ * This program is free software: you can redistribute it and/or modify it under
+ * the terms of the GNU General Public License as published by the Free Software
+ * Foundation, either version 2 of the License, or (at your option) any later
+ * version. See http://www.gnu.org/copyleft/gpl.html the full text of the
+ * license.
  */
 
 #include <langinfo.h>
@@ -23,8 +16,7 @@
 #include <stdio.h>
 
 #include "mp-serializer.h"
-
-#include "math-enums.h"
+#include "mp-enums.h"
 
 enum {
     PROP_0,
@@ -34,8 +26,6 @@ enum {
     PROP_BASE,
 };
 
-static GType number_format_type;
-
 struct MpSerializerPrivate
 {
     gint accuracy;            /* Number of digits to show */
@@ -108,7 +98,7 @@ mp_cast_to_string_real(MpSerializer *serializer, const MPNumber *x, int base, gb
         g_string_prepend_c(string, d < 16 ? digits[d] : '?');
 
         i++;
-        if (serializer->priv->show_tsep && i == serializer->priv->tsep_count) {
+        if (serializer->priv->base == 10 && serializer->priv->show_tsep && i == serializer->priv->tsep_count) {
             g_string_prepend_unichar(string, serializer->priv->tsep);
             i = 0;
         }
@@ -223,17 +213,12 @@ mp_cast_to_string(MpSerializer *serializer, const MPNumber *x)
 }
 
 
-static gchar *
-mp_cast_to_exponential_string(MpSerializer *serializer, const MPNumber *x, gboolean eng_format)
+static int
+mp_cast_to_exponential_string_real(MpSerializer *serializer, const MPNumber *x, GString *string, gboolean eng_format)
 {
-    gchar *fixed, *c;
+    gchar *fixed;
     MPNumber t, z, base, base3, base10, base10inv, mantissa;
     int exponent = 0;
-    GString *string;
-    gchar *result;
-    const gchar *super_digits[] = {"⁰", "¹", "²", "³", "⁴", "⁵", "⁶", "⁷", "⁸", "⁹"};
-
-    string = g_string_sized_new(1024);
 
     mp_abs(x, &z);
     if (mp_is_negative(x))
@@ -273,19 +258,83 @@ mp_cast_to_exponential_string(MpSerializer *serializer, const MPNumber *x, gbool
     fixed = mp_cast_to_string(serializer, &mantissa);
     g_string_append(string, fixed);
     g_free(fixed);
-    if (exponent != 0) {
-        gchar *super_value;
+  
+    return exponent;
+}
 
-        g_string_append_printf(string, "×10"); // FIXME: Use the current base
-        if (exponent < 0) {
-            exponent = -exponent;
-            g_string_append(string, "⁻");
+
+static void
+append_exponent(GString *string, int exponent)
+{
+    const gchar *super_digits[] = {"⁰", "¹", "²", "³", "⁴", "⁵", "⁶", "⁷", "⁸", "⁹"};
+    gchar *super_value, *c;
+  
+    if (exponent == 0)
+        return;
+
+    g_string_append_printf(string, "×10"); // FIXME: Use the current base
+    if (exponent < 0) {
+        exponent = -exponent;
+        g_string_append(string, "⁻");
+    }
+
+    super_value = g_strdup_printf("%d", exponent);
+    for (c = super_value; *c; c++)
+        g_string_append(string, super_digits[*c - '0']);
+    g_free (super_value);
+}
+
+
+static gchar *
+mp_cast_to_exponential_string(MpSerializer *serializer, const MPNumber *x, gboolean eng_format)
+{
+    GString *string;
+    MPNumber x_real;
+    gchar *result;
+    int exponent;
+
+    string = g_string_sized_new(1024);
+
+    mp_real_component(x, &x_real);
+    exponent = mp_cast_to_exponential_string_real(serializer, &x_real, string, eng_format);
+    append_exponent(string, exponent);
+
+    if (mp_is_complex(x)) {
+        GString *s;
+        gboolean force_sign = TRUE;
+        MPNumber x_im;
+
+        mp_imaginary_component(x, &x_im);
+
+        if (strcmp(string->str, "0") == 0) {
+            g_string_assign(string, "");
+            force_sign = FALSE;
         }
 
-        super_value = g_strdup_printf("%d", exponent);
-        for (c = super_value; *c; c++)
-            g_string_append(string, super_digits[*c - '0']);
-        g_free (super_value);
+        s = g_string_sized_new(1024);
+        exponent = mp_cast_to_exponential_string_real(serializer, &x_im, s, eng_format);
+        if (strcmp(s->str, "0") == 0 || strcmp(s->str, "+0") == 0 || strcmp(s->str, "−0") == 0) {
+            /* Ignore */
+        }
+        else if (strcmp(s->str, "1") == 0) {
+            g_string_append(string, "i");
+        }
+        else if (strcmp(s->str, "+1") == 0) {
+            g_string_append(string, "+i");
+        }
+        else if (strcmp(s->str, "−1") == 0) {
+            g_string_append(string, "−i");
+        }
+        else {
+            if (strcmp(s->str, "+0") == 0)
+                g_string_append(string, "+");
+            else if (strcmp(s->str, "0") != 0)
+                g_string_append(string, s->str);
+
+            g_string_append(string, "i");
+        }
+        g_string_free(s, TRUE);
+        append_exponent(string, exponent);
     }
 
     result = g_strndup(string->str, string->len + 1);
@@ -304,7 +353,7 @@ mp_serializer_to_string(MpSerializer *serializer, const MPNumber *x)
     case MP_DISPLAY_FORMAT_AUTOMATIC:
         s0 = mp_cast_to_string(serializer, x);
         s1 = mp_cast_to_exponential_string(serializer, x, FALSE);
-        if (g_utf8_strlen(s0, -1) < g_utf8_strlen(s1, -1))
+        if (serializer->priv->base != 10 || g_utf8_strlen(s0, -1) < g_utf8_strlen(s1, -1))
         {
             g_free(s1);
             return s0;
@@ -314,7 +363,7 @@ mp_serializer_to_string(MpSerializer *serializer, const MPNumber *x)
             g_free(s0);
             return s1;
         }
-      break;
+        break;
     case MP_DISPLAY_FORMAT_FIXED:
         return mp_cast_to_string(serializer, x);
     case MP_DISPLAY_FORMAT_SCIENTIFIC:
@@ -496,8 +545,6 @@ mp_serializer_class_init(MpSerializerClass *klass)
 
     g_type_class_add_private(klass, sizeof(MpSerializerPrivate));
 
-    number_format_type = math_mp_display_format_get_type();
-
     g_object_class_install_property(object_class,
                                     PROP_SHOW_THOUSANDS_SEPARATORS,
                                     g_param_spec_boolean("show-thousands-separators",
@@ -517,7 +564,7 @@ mp_serializer_class_init(MpSerializerClass *klass)
                                     g_param_spec_enum("number-format",
                                                       "number-format",
                                                       "Display format",
-                                                      number_format_type,
+                                                      math_mp_display_format_get_type(),
                                                       MP_DISPLAY_FORMAT_AUTOMATIC,
                                                       G_PARAM_READWRITE));
     g_object_class_install_property(object_class,
